# Copyright 2016 Catalyst IT Limited
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg

ServiceAvailableGroup = [cfg.BoolOpt('mistral',
                                     default=True,
                                     help="Whether or not Mistral is expected"
                                          " to be available")]

service_available_group = cfg.OptGroup(name="service_available",
                                       title="Available OpenStack Services")

mistral_api_group = cfg.OptGroup(name="mistral_api",
                                 title="Mistral Api Service Options")

MistralApiGroup = [cfg.BoolOpt("service_api_supported",
                               default=False,
                               help="Whether or not services api "
                                    "is available")]
